/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("./types");
const collections_1 = require("./collections");
function newNode(data) {
    return {
        data: data,
        incoming: Object.create(null),
        outgoing: Object.create(null)
    };
}
class Graph {
    constructor(_hashFn) {
        this._hashFn = _hashFn;
        this._nodes = Object.create(null);
        // empty
    }
    roots() {
        const ret = [];
        collections_1.forEach(this._nodes, entry => {
            if (types_1.isEmptyObject(entry.value.outgoing)) {
                ret.push(entry.value);
            }
        });
        return ret;
    }
    traverse(start, inwards, callback) {
        const startNode = this.lookup(start);
        if (!startNode) {
            return;
        }
        this._traverse(startNode, inwards, Object.create(null), callback);
    }
    _traverse(node, inwards, seen, callback) {
        const key = this._hashFn(node.data);
        if (collections_1.contains(seen, key)) {
            return;
        }
        seen[key] = true;
        callback(node.data);
        const nodes = inwards ? node.outgoing : node.incoming;
        collections_1.forEach(nodes, (entry) => this._traverse(entry.value, inwards, seen, callback));
    }
    insertEdge(from, to) {
        const fromNode = this.lookupOrInsertNode(from), toNode = this.lookupOrInsertNode(to);
        fromNode.outgoing[this._hashFn(to)] = toNode;
        toNode.incoming[this._hashFn(from)] = fromNode;
    }
    removeNode(data) {
        const key = this._hashFn(data);
        delete this._nodes[key];
        collections_1.forEach(this._nodes, (entry) => {
            delete entry.value.outgoing[key];
            delete entry.value.incoming[key];
        });
    }
    lookupOrInsertNode(data) {
        const key = this._hashFn(data);
        let node = collections_1.lookup(this._nodes, key);
        if (!node) {
            node = newNode(data);
            this._nodes[key] = node;
        }
        return node;
    }
    lookup(data) {
        return collections_1.lookup(this._nodes, this._hashFn(data));
    }
    get length() {
        return Object.keys(this._nodes).length;
    }
    toString() {
        const data = [];
        collections_1.forEach(this._nodes, entry => {
            data.push(`${entry.key}, (incoming)[${Object.keys(entry.value.incoming).join(", ")}], (outgoing)[${Object.keys(entry.value.outgoing).join(",")}]`);
        });
        return data.join("\n");
    }
}
exports.Graph = Graph;
